@@ -2,7 +2,7 @@
 //
 // The intrinsic Forth word list for kForth
 //
-// Copyright (c) 2008--2022 Krishna Myneni,
+// Copyright (c) 2008--2024 Krishna Myneni,
 //   <krishna.myneni@ccreweb.org> 
 //
 // This software is provided under the terms of the GNU
@@ -31,19 +31,28 @@ WordTemplate ForthWords[] =
     { "SET-CURRENT", OP_SETCURRENT, NONDEFERRED },
     { "GET-ORDER",   OP_GETORDER,      0 },
     { "SEARCH-WORDLIST", OP_SEARCHWORDLIST, 0 },
+    { "TRAVERSE-WORDLIST", OP_TRAVERSE_WORDLIST, 0 },
+    { "NAME>STRING", OP_NAME2STRING,          0 },
+    { "NAME>INTERPRET", OP_NAME2INTERPRET,    0 },
+    { "NAME>COMPILE", OP_NAME2COMPILE,        0 },
+    { "MY-NAME",   OP_MYNAME,                 0 },
     { "ALSO",      OP_ALSO,         NONDEFERRED },
     { "ONLY",      OP_ONLY,         NONDEFERRED },
     { "PREVIOUS",  OP_PREVIOUS,     NONDEFERRED },
     { "VOCABULARY", OP_VOCABULARY,  NONDEFERRED },
     { "ASSEMBLER", OP_ASSEMBLER,    NONDEFERRED },
     { "PARSE",     OP_PARSE,        NONDEFERRED },
+    { "PARSE-NAME", OP_PARSENAME,   NONDEFERRED },
     { "WORD",      OP_WORD,         NONDEFERRED },
     { "FIND",      OP_FIND,         0 },
+    { "FIND-NAME-IN", OP_FIND_NAME_IN, 0 },
+    { "FIND-NAME", OP_FIND_NAME,    0 },
     { "'",         OP_TICK,         NONDEFERRED },
     { "[']",       OP_BRACKETTICK,  IMMEDIATE },
     { "[DEFINED]", OP_DEFINED,      IMMEDIATE | NONDEFERRED },
     { "[UNDEFINED]", OP_UNDEFINED,  IMMEDIATE | NONDEFERRED },
     { "COMPILE,",  OP_COMPILECOMMA, NONDEFERRED },
+    { "COMPILE-NAME", OP_COMPILENAME, NONDEFERRED },
     { "POSTPONE",  OP_POSTPONE,     IMMEDIATE },
     { "[",         OP_LBRACKET,     IMMEDIATE },
     { "]",         OP_RBRACKET,     NONDEFERRED },
@@ -51,12 +60,16 @@ WordTemplate ForthWords[] =
     { "CREATE",    OP_CREATE,       NONDEFERRED },
     { "DOES>",     OP_DOES,         0 },
     { ">BODY",     OP_TOBODY,       0 },
+    { "SYNONYM",   OP_SYNONYM,      NONDEFERRED },
     { "ALIAS",     OP_ALIAS,        NONDEFERRED },
     { "FORGET",    OP_FORGET,       NONDEFERRED },
     { "COLD",      OP_COLD,         NONDEFERRED },
     { ":NONAME",   OP_NONAME,       NONDEFERRED },
     { ":",         OP_COLON,        NONDEFERRED },
     { ";",         OP_SEMICOLON,    IMMEDIATE },
+#ifndef _WIN32_
+    { "USED",      OP_USED,         0 },
+#endif
     { "ALLOCATE",  OP_ALLOCATE,     0 },     
     { "FREE",      OP_FREE,         0 },
     { "RESIZE",    OP_RESIZE,       0 },
@@ -88,12 +101,20 @@ WordTemplate ForthWords[] =
     { "CELLS",     OP_CELLS,        0 },
     { "CELL+",     OP_CELLPLUS,     0 },
     { "CHAR+",     OP_INC,          0 },
-    { "FLOATS",    OP_DFLOATS,      0 },
-    { "FLOAT+",    OP_DFLOATPLUS,   0 },
     { "DFLOATS",   OP_DFLOATS,      0 },
     { "DFLOAT+",   OP_DFLOATPLUS,   0 },
+#ifdef __NO_FPSTACK__
+    { "FLOATS",    OP_DFLOATS,      0 },
+    { "FLOAT+",    OP_DFLOATPLUS,   0 },
     { "SFLOATS",   OP_CELLS,        0 },
     { "SFLOAT+",   OP_CELLPLUS,     0 },
+#else
+    { "FLOATS",    OP_FLOATS,       0 },
+    { "FLOAT+",    OP_FLOATPLUS,    0 },
+    { "SFLOATS",   OP_SFLOATS,      0 },
+    { "SFLOAT+",   OP_SFLOATPLUS,   0 },
+#endif
+    { "}}",        OP_FSL_MAT_ADDR, 0 },
     { "?",         OP_QUESTION,     0 },
     { "@",         OP_FETCH,        0 },
     { "!",         OP_STORE,        0 },
@@ -119,6 +140,10 @@ WordTemplate ForthWords[] =
     { "SP!",       OP_SPSTORE,      0 },
     { "RP@",       OP_RPFETCH,      0 },
     { "RP!",       OP_RPSTORE,      0 },
+#ifndef __NO_FPSTACK__
+    { "FP@",       OP_FPFETCH,      0 },
+    { "FP!",       OP_FPSTORE,      0 },
+#endif
     { ">R",        OP_PUSH,         0 },
     { "R>",        OP_POP,          0 },
     { "R@",        OP_RFETCH,       0 },
@@ -142,6 +167,9 @@ WordTemplate ForthWords[] =
     { "2OVER",     OP_2OVER,        0 },
     { "2ROT",      OP_2ROT,         0 },
     { "DEPTH",     OP_DEPTH,        0 },
+#ifndef __NO_FPSTACK__
+    { "FDEPTH",    OP_FDEPTH,       0 },
+#endif
     { "BASE",      OP_BASE,         0 },
     { "BINARY",    OP_BINARY,       NONDEFERRED },
     { "DECIMAL",   OP_DECIMAL,      NONDEFERRED },
@@ -182,6 +210,7 @@ WordTemplate ForthWords[] =
     { "ABORT\x22", OP_ABORTQUOTE,   IMMEDIATE },
     { "VMTHROW",   OP_VMTHROW,      0 },
     { "USLEEP",    OP_USLEEP,       0 },
+    { "EXECUTE-BC", OP_EXECUTE_BC,  0 },
     { "EXECUTE",   OP_EXECUTE,      0 },
     { "CALL",      OP_CALL,         0 },
     { "SYSTEM",    OP_SYSTEM,       0 },
@@ -238,6 +267,9 @@ WordTemplate ForthWords[] =
     { "FS.",       OP_FSDOT,        0 },
     { ".\x22",     OP_DOTQUOTE,     IMMEDIATE },
     { ".S",        OP_DOTS,         0 },
+#ifndef __NO_FPSTACK__
+    { "F.S",       OP_FDOTS,        0 },
+#endif
     { "CR",        OP_CR,           0 },
     { "SPACES",    OP_SPACES,       0 },
     { "EMIT",      OP_EMIT,         0 },
@@ -279,11 +311,17 @@ WordTemplate ForthWords[] =
     { "XOR",       OP_XOR,          0 },
     { "NOT",       OP_NOT,          0 },
     { "INVERT",    OP_NOT,          0 },
+    { "BOOLEAN?",  OP_BOOLEAN_QUERY, 0 },
+    { ".NOT.",     OP_BOOL_NOT,     0 },
+    { ".AND.",     OP_BOOL_AND,     0 },
+    { ".OR.",      OP_BOOL_OR,      0 },
+    { ".XOR.",     OP_BOOL_XOR,     0 },
     { "LSHIFT",    OP_LSHIFT,       0 },
     { "RSHIFT",    OP_RSHIFT,       0 },
     { "+",         OP_ADD,          0 },
     { "-",         OP_SUB,          0 },
     { "*",         OP_MUL,          0 },
+    { "*+",        OP_STARPLUS,     0 },
     { "/",         OP_DIV,          0 },
     { "MOD",       OP_MOD,          0 },
     { "/MOD",      OP_SLASHMOD,     0 },
@@ -330,13 +368,22 @@ WordTemplate ForthWords[] =
     { "MOVE",      OP_MOVE,         0 },
     { "CMOVE",     OP_CMOVE,        0 },
     { "CMOVE>",    OP_CMOVEFROM,    0 },
+#ifdef __NO_FPSTACK__
     { "FDUP",      OP_2DUP,         0 },
     { "FDROP",     OP_2DROP,        0 },
-    { "F2DROP",    OP_F2DROP,       0 },
-    { "F2DUP",     OP_F2DUP,        0 },
     { "FSWAP",     OP_2SWAP,        0 },
     { "FOVER",     OP_2OVER,        0 },
     { "FROT",      OP_2ROT,         0 },
+#else
+    { "FDUP",      OP_FDUP,         0 },
+    { "FDROP",     OP_FDROP,        0 },
+    { "FSWAP",     OP_FSWAP,        0 },
+    { "FOVER",     OP_FOVER,        0 },
+    { "FROT",      OP_FROT,         0 },
+    { "FPICK",     OP_FPICK,        0 },
+#endif
+    { "F2DROP",    OP_F2DROP,       0 },
+    { "F2DUP",     OP_F2DUP,        0 },
     { "F=",        OP_FEQ,          0 },
     { "F<>",       OP_FNE,          0 },
     { "F<",        OP_FLT,          0 },
@@ -351,7 +398,9 @@ WordTemplate ForthWords[] =
     { "F*",        OP_FMUL,         0 },
     { "F/",        OP_FDIV,         0 },
     { "F**",       OP_FPOW,         0 },
+    { "F+!",       OP_FPLUSSTORE,   0 },
     { "FSQRT",     OP_FSQRT,        0 },
+    { "FSQUARE",   OP_FSQUARE,      0 },
     { "FABS",      OP_FABS,         0 },
     { "FNEGATE",   OP_FNEG,         0 },
     { "FLOOR",     OP_FLOOR,        0 },
@@ -379,6 +428,7 @@ WordTemplate ForthWords[] =
     { "FLNP1",     OP_FLNP1,        0 },
     { "FEXP",      OP_FEXP,         0 },
     { "FEXPM1",    OP_FEXPM1,       0 },
+    { "PI",        OP_PI,           0 },
     { "DEG>RAD",   OP_DEGTORAD,     0 },
     { "RAD>DEG",   OP_RADTODEG,     0 },
     { "S>D",       OP_STOD,         0 },
