@@ -3,7 +3,7 @@
 // Useful macros for kForth C and C++ source files, following
 // the convention established by DNW in vm-osxppc.s
 //
-// Copyright (c) 2009--2022, Krishna Myneni
+// Copyright (c) 2009--2021, Krishna Myneni
 //   <krishna.myneni@ccreweb.org>
 //
 // This software is provided under the terms of the GNU
@@ -14,6 +14,11 @@
 #define DEC_DSP   --GlobalSp;
 #define TOS       (*GlobalSp)
 
+#ifndef __NO_FPSTACK__
+#define INC_FSP   (GlobalFp = (void*)((byte*)GlobalFp + FpSize));
+#define DEC_FSP   (GlobalFp = (void*)((byte*)GlobalFp - FpSize));
+#endif
+
 #ifndef __FAST__
 
 #define INC_DTSP  ++GlobalTp;
