@@ -3,7 +3,7 @@ vmc.c
 
   C portion of the kForth Virtual Machine
 
-  Copyright (c) 1998--2022 Krishna Myneni, 
+  Copyright (c) 1998--2025 Krishna Myneni, 
   <krishna.myneni@ccreweb.org>
 
   This software is provided under the terms of the GNU
@@ -11,9 +11,7 @@ vmc.c
 
 */
 
-#ifndef _WIN32_
 #define _GNU_SOURCE
-#endif
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -31,23 +29,26 @@ vmc.c
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
+#include <malloc.h>
 #include <math.h>
 #include "fbc.h"
 #include "VMerrors.h"
 #include "kfmacros.h"
 
-#define WSIZE 4
+#define WSIZE 8
 #define TRUE -1
 #define FALSE 0
 
 #define byte unsigned char
 
-/*  Provided by ForthVM.cpp  */
-extern int* GlobalSp;
+//  Provided by ForthVM.cpp
+extern long int* GlobalSp;
+extern void* GlobalFp;
 extern byte* GlobalIp;
-extern int* GlobalRp;
-extern int* BottomOfStack;
-extern int* BottomOfReturnStack;
+extern long int* GlobalRp;
+extern long int* BottomOfStack;
+extern long int* BottomOfReturnStack;
+extern long int FpSize;
 #ifndef __FAST__
 extern byte* GlobalTp;
 extern byte* GlobalRtp;
@@ -57,40 +58,37 @@ extern byte* BottomOfReturnTypeStack;
 extern int CPP_bye();
 
 // Provided by vmxx-common.s
-extern int Base;
-extern int State;
+extern long int Base;
+extern long int State;
 extern char* pTIB;
-extern int NumberCount;
-extern int  JumpTable[];
+extern long int NumberCount;
+extern long int JumpTable[];
 extern char WordBuf[];
 extern char TIB[];
 extern char NumberBuf[];
 
-/*  Provided by vmxx.s/vmxx-fast.s  */
-extern int L_dnegate();
-extern int L_dplus();
-extern int L_dminus();
-extern int L_udmstar();
-extern int L_utmslash();
-extern int L_uddivmod();
-extern int L_quit();
-extern int L_abort();
-extern int vm(byte*);
+//  Provided by vmxx.s/vmxx-fast.s
+int L_dnegate();
+int L_dplus();
+int L_dminus();
+int L_udmstar();
+int L_uddivmod();
+int L_utmslash();
+int L_quit();
+int L_abort();
+int vm(byte*);
 
-#ifdef _WIN32_
-unsigned long int ForthStartTime;
-#else
 struct timeval ForthStartTime;
 struct termios tios0;
-char key_query_char = 0;
-#endif
+struct mallinfo ForthStartMem;
 double* pf;
 double f;
 char temp_str[256];
+char key_query_char = 0;
 
 /*  signal dispatch table  */
 
-void* signal_xtmap [32] =
+void** signal_xtmap [32] =
 {
     NULL,              //  1  SIGHUP, Hangup
     NULL,              //  2  SIGINT, Interrupt
@@ -128,35 +126,6 @@ void* signal_xtmap [32] =
 
 static void forth_signal_handler (int, siginfo_t*, void*); 
 
-#define DOUBLE_FUNC(x)   pf = (double*)(GlobalSp+1); *pf=x(*pf);
-  
-int C_ftan  () { DOUBLE_FUNC(tan)  return 0; }
-int C_facos () { DOUBLE_FUNC(acos) return 0; }
-int C_fasin () { DOUBLE_FUNC(asin) return 0; }
-int C_fatan () { DOUBLE_FUNC(atan) return 0; }
-int C_fsinh () { DOUBLE_FUNC(sinh) return 0; }
-int C_fcosh () { DOUBLE_FUNC(cosh) return 0; }
-int C_ftanh () { DOUBLE_FUNC(tanh) return 0; }
-int C_fasinh () { DOUBLE_FUNC(asinh) return 0; }
-int C_facosh () { DOUBLE_FUNC(acosh) return 0; }
-int C_fatanh () { DOUBLE_FUNC(atanh) return 0; }
-int C_fexp  () { DOUBLE_FUNC(exp)   return 0; }
-int C_fexpm1() { DOUBLE_FUNC(expm1) return 0; }
-int C_fln   () { DOUBLE_FUNC(log)   return 0; }
-int C_flnp1 () { DOUBLE_FUNC(log1p) return 0; }
-int C_flog  () { DOUBLE_FUNC(log10) return 0; }
-#ifndef _WIN32_
-int C_falog () { DOUBLE_FUNC(exp10) return 0; }
-#else
-int C_falog ()
-{
-     pf = (double*)(GlobalSp + 1);
-     f = *pf;
-     *pf = pow(10., f);
-     return 0;
-}
-#endif
-
 // powA  is copied from the source of the function pow() in paranoia.c,
 //   at  http://www.math.utah.edu/~beebe/software/ieee/ 
 double powA(double x, double y) /* return x ^ y (exponentiation) */
@@ -186,36 +155,55 @@ double powA(double x, double y) /* return x ^ y (exponentiation) */
     return ldexp(xy, ey);
 } 
 
+// #define DOUBLE_FUNC(x)   pf = (double*)((byte*)GlobalFp+FpSize); *pf=x(*pf);
+
+// int C_fsin  () { DOUBLE_FUNC(sin)  return 0; }
+// int C_fcos  () { DOUBLE_FUNC(cos)  return 0; }
+// int C_ftan  () { DOUBLE_FUNC(tan)  return 0; }
+// int C_facos () { DOUBLE_FUNC(acos) return 0; }
+// int C_fasin () { DOUBLE_FUNC(asin) return 0; }
+// int C_fatan () { DOUBLE_FUNC(atan) return 0; }
+// int C_fsinh () { DOUBLE_FUNC(sinh) return 0; }
+// int C_fcosh () { DOUBLE_FUNC(cosh) return 0; }
+// int C_ftanh () { DOUBLE_FUNC(tanh) return 0; }
+// int C_fasinh () { DOUBLE_FUNC(asinh) return 0; }
+// int C_facosh () { DOUBLE_FUNC(acosh) return 0; }
+// int C_fatanh () { DOUBLE_FUNC(atanh) return 0; }
+// int C_fexp  () { DOUBLE_FUNC(exp)   return 0; }
+// int C_fexpm1() { DOUBLE_FUNC(expm1) return 0; }
+// int C_fln   () { DOUBLE_FUNC(log)   return 0; }
+// int C_flnp1 () { DOUBLE_FUNC(log1p) return 0; }
+// int C_flog  () { DOUBLE_FUNC(log10) return 0; }
+// int C_falog () { DOUBLE_FUNC(exp10) return 0; }
+// int C_floor () { DOUBLE_FUNC(floor) return 0; }
+
 int C_fpow ()
 {
-	pf = (double*)(GlobalSp + 1);
+	pf = (double*)((byte*) GlobalFp + FpSize);
 	f = *pf;
 	++pf;
 	*pf = powA (*pf, f);
-	GlobalSp += 2;
-	INC2_DTSP
+	INC_FSP
 	return 0;
 }				
 
 int C_fmin ()
 {
-	pf = (double*)(GlobalSp + 1);
+	pf = (double*)((byte*) GlobalFp + FpSize);
 	f = *pf;
 	++pf;
 	if (f < *pf) *pf = f;
-	GlobalSp += 2;
-	INC2_DTSP
+	INC_FSP
 	return 0;
 }
 
 int C_fmax ()
 {
-	pf = (double*)(GlobalSp + 1);
+	pf = (double*)((byte*) GlobalFp + FpSize);
 	f = *pf;
 	++pf;
 	if (f > *pf) *pf = f;
-	GlobalSp += 2;
-	INC2_DTSP
+	INC_FSP
 	return 0;
 }
 
@@ -243,7 +231,8 @@ int C_lseek ()
 {
   /* stack: ( fd offset mode -- error | set file position in fd ) */
 
-  int fd, offset, mode;
+  int fd, mode;
+  unsigned long int offset;
   DROP
   mode = TOS;
   DROP
@@ -275,12 +264,12 @@ int C_read ()
   void* buf;
 
   DROP
-  count = TOS;
+  count = (int)(TOS);
   DROP
   CHK_ADDR
   buf = *((void**)GlobalSp);
   DROP
-  fd = TOS;
+  fd = (int)(TOS);
   PUSH_IVAL( read (fd, buf, count) )
   return 0;
 }
@@ -308,12 +297,14 @@ int C_write ()
 // word, FLUSH-FILE (Forth 94/Forth 2012)
 int C_fsync ()
 {
+  /* stack: ( fd -- ior )  */
   int fd;
   DROP
   fd = TOS;
   PUSH_IVAL( fsync(fd) )
   return 0;
 }
+  
 
 int C_ioctl ()
 {
@@ -336,8 +327,8 @@ int C_ioctl ()
 int C_dlopen ()
 {
    /* stack: ( azLibName flag -- handle | NULL) */
-   unsigned flags;
-   int handle;
+   unsigned long flags;
+   long int handle;
    char *pLibName;
 
    DROP
@@ -346,7 +337,7 @@ int C_dlopen ()
    CHK_ADDR
    pLibName = *((char**) GlobalSp);  // pointer to a null-terminated string
 
-   handle = (int) dlopen((const char*) pLibName, flags);
+   handle = (long int) dlopen((const char*) pLibName, flags);
    PUSH_IVAL(handle)
    return 0;
 }
@@ -356,14 +347,14 @@ int C_dlerror ()
    /* stack: ( -- addrz) ; Returns address of null-terminated string*/
    char *errMsg;
    errMsg = dlerror();
-   PUSH_ADDR((int) errMsg)
+   PUSH_ADDR((long int) errMsg)
    return 0;
 }
 
 int C_dlsym ()
 {
     /* stack: ( handle azsymbol -- addr ) */
-    int handle;
+    long int handle;
     char *pSymbol;
     void *pSymAddr;
 
@@ -374,14 +365,14 @@ int C_dlsym ()
     handle = TOS;
 
     pSymAddr = dlsym((void*)handle, (const char*) pSymbol);
-    PUSH_ADDR((int) pSymAddr)
+    PUSH_ADDR((long int) pSymAddr)
     return 0;
 }
 
 int C_dlclose ()
 {
     /* stack: ( handle -- error | 0) */
-    int handle;
+    long int handle;
     INC_DSP
     handle = TOS;
     TOS = dlclose((void*)handle);
@@ -489,7 +480,7 @@ int C_accept ()
   /* stack: ( a n1 -- n2 | wait for n characters to be received ) */
 
   char *cp, *cpstart, *bksp = "\010 \010";
-  int n1, n2, nr;
+  long int n1, n2, nr;
   struct termios t1, t2;
 
   DROP
@@ -541,9 +532,9 @@ int C_accept ()
 char* strupr (char* p)
 {
 /* convert string to upper case  */
-  char* p0 = p;
-  while (*p) {*p = toupper(*p); ++p;}
-  return p0;
+  char* cp = p;
+  while (*cp) {*cp = toupper(*cp); ++cp;}
+  return p;
 }
 
 char* ExtractName (char* str, char* name)
@@ -579,7 +570,12 @@ Check the string token to see if it is an LMI style floating point
 number; if so set the value of *p and return True, otherwise
 return False.
 */
-    char *pStr = token;
+    char s[256];
+    char *pStr = &s[0];
+    char *pEnd;
+    int f = FALSE;
+
+    strcpy(s, token);
 
     if (strchr(pStr, 'E'))
     {
@@ -593,13 +589,19 @@ return False.
             /* LMI Forth style */
 
             --pStr;
-            if (*pStr == 'E') *pStr = '\0';
-            *p = atof(token);
-            return TRUE;
+            if ((*pStr == '+') || (*pStr == '-')) {
+	      *pStr = '\0';
+              --pStr;
+	    }
+	    if (pStr > &s[0]) {
+              if (*pStr == 'E') *pStr = '\0';
+	    }
+            *p = strtod(s, &pEnd);
+             if (*pEnd == 0) f = TRUE;
         }
     }
 
-    return FALSE;
+    return f;
 }
 /*----------------------------------------------------------*/
 
@@ -612,18 +614,17 @@ int isBaseDigit (int c)
 }
 /*---------------------------------------------------------*/
 
-int IsInt (char* token, int* p)
+int IsInt (char* token, long int* p)
 {
 /* Check the string token to see if it is an integer number;
    if so set the value of *p and return True, otherwise return False. */
 
-  int b = FALSE, sign = FALSE;
-  unsigned u = 0;
+  int b = FALSE;
+  unsigned long u = 0;
   char *pStr = token, *endp;
 
   if ((*pStr == '-') || isBaseDigit(*pStr))
     {
-      if (*pStr == '-') {sign = TRUE;}
       ++pStr;
       while (isBaseDigit(*pStr))	    
 	{
@@ -657,7 +658,7 @@ int C_word ()
     }
   if (*pTIB)
     {
-      int count = 0;
+      long int count = 0;
       while (*pTIB)
 	{
 	  if (*pTIB == delim) break;
@@ -672,26 +673,26 @@ int C_word ()
     {
       *WordBuf = 0;
     }
-  PUSH_ADDR((int) WordBuf)
+  PUSH_ADDR((long int) WordBuf)
   return 0;
 }
 
 // PARSE  ( char "ccc<char>" -- c-addr u )
 // Parse text delimited by char; return string address and count.
-// Forth-94 Core Extensions wordset 6.2.2008
+// Forth 2012 Core Extensions wordset 6.2.2008
 int C_parse ()
 {
   DROP
   char delim = TOS;
-  char *cp = pTIB;
-  int count = 0;
+  char *cp = pTIB; 
+  long int count = 0;
   if (*pTIB)
     {
 
       while (*pTIB)
 	{
 	  if (*pTIB == delim) break;
-	  ++pTIB;
+	  ++pTIB; 
 	  ++count;
 	}
       if (*pTIB) ++pTIB;  /* consume the delimiter */
@@ -700,12 +701,34 @@ int C_parse ()
   PUSH_IVAL(count)
   return 0;
 }
+
+// PARSE-NAME  ( "<spaces>name<space>" -- c-addr u )
+// Skip leading whitespace and parse name delimited by
+// whitespace character; return string address and count.
+// Forth 2012 Core Extensions wordset 6.2.2020
+int C_parsename ()
+{
+  long int count = 0;
+  char *cp = pTIB;
+  const char* delim = "\t ";
+  // Skip leading delimiters
+  while (*pTIB && strchr(delim, *pTIB)) ++pTIB;
+  cp = pTIB;
+  while (*pTIB && (strchr(delim, *pTIB) == NULL)) {
+    ++pTIB;
+    ++count;
+  }
+  PUSH_ADDR((long int) cp)
+  PUSH_IVAL(count)
+  return 0;
+}
+
 /*----------------------------------------------------------*/
 
 int C_trailing ()
 {
   /* stack: ( a n1 -- a n2 | adjust count n1 to remove trailing spaces ) */
-  int n1;
+  long int n1;
   char *cp;
   DROP
   n1 = TOS;
@@ -749,13 +772,13 @@ int C_sharp()
   uq1  = TOS;   /* quotient: uq1,uq2 */
   INC_DSP
   uq2  = TOS;
-  INC_DSP
-  urem = TOS;   /* remainder */
+  INC_DSP  
+  urem = TOS;  /* remainder */
   TOS = uq2;
   DEC_DSP
   TOS = uq1;
   DEC_DSP
-	
+
   ch = (urem < 10) ? (urem + 48) : (urem + 55);
   ++NumberCount;
   NumberBuf[255 - NumberCount] = ch;
@@ -768,7 +791,7 @@ int C_sharps()
 {
   /* stack: ( ud -- 0 0 | finish converting all digits of ud ) */
 
-  unsigned int u1=1, u2=0;
+  unsigned long int u1=1, u2=0;
 
   while (u1 | u2)
     {
@@ -795,7 +818,7 @@ int C_sign()
 {
   /* stack: ( n -- | insert sign into number string if n < 0 ) */
   DROP
-  int n = TOS;
+  long int n = TOS;
   if (n < 0)
     {
       ++NumberCount;
@@ -811,7 +834,7 @@ int C_sharpbracket()
 
   DROP
   DROP
-  PUSH_ADDR( (int) (NumberBuf + 255 - NumberCount) )
+  PUSH_ADDR( (long int) (NumberBuf + 255 - NumberCount) )
   PUSH_IVAL(NumberCount)
   return 0;
 }
@@ -821,10 +844,10 @@ int C_tonumber ()
 {
   /* stack: ( ud1 a1 u1 -- ud2 a2 u2 | translate characters into ud number ) */
 
-  unsigned i, ulen, uc;
+  unsigned long i, ulen, uc;
   int c;
   char *cp;
-  ulen = (unsigned) *(GlobalSp + 1);
+  ulen = (unsigned long) *(GlobalSp + 1);
   if (ulen == 0) return 0;
   uc = ulen;
   DROP
@@ -856,7 +879,7 @@ int C_tonumber ()
         --uc; ++cp;
   }
 
-  TOS = (int) cp;
+  TOS = (long int) cp;
   DEC_DSP
   TOS = uc;
   DEC_DSP
@@ -867,53 +890,14 @@ int C_tonumber ()
 }
 /*-----------------------------------------------------------*/
  
-int C_numberquery ()
-{
-  /* stack: ( ^str -- d b | translate characters into number using current base ) */
-
-  char *pStr;
-  int b, sign, nc;
-
-  b = FALSE;
-  sign = FALSE;
-
-  DROP
-  if (GlobalSp > BottomOfStack) return E_V_STK_UNDERFLOW;
-  CHK_ADDR
-  pStr = *((char**)GlobalSp);
-  PUSH_IVAL(0)
-  PUSH_IVAL(0)
-  nc = *pStr;
-  ++pStr;
-
-  if (*pStr == '-') {
-    sign = TRUE; ++pStr; --nc;
-  }
-  if (nc > 0) {
-        PUSH_ADDR((int) pStr)
-        PUSH_IVAL(nc)
-        C_tonumber();
-	DROP
-        b = TOS;
-	DROP
-	b = (b == 0) ? TRUE : FALSE ;
-  }
-
-  if (sign) L_dnegate();
-
-  PUSH_IVAL(b)
-  return 0;
-}
-/*----------------------------------------------------------*/
-
 int C_tofloat ()
 {
   /* stack: ( a u -- f true | false ; convert string to floating point number ) */
 
   char s[256], *cp;
   double f;
-  unsigned nc, u;
-  int b;
+  unsigned long nc, u;
+  long int b;
 
   DROP
   nc = TOS;
@@ -975,24 +959,59 @@ int C_tofloat ()
     
 
   if (b) {
-      DEC_DSP
-      *((double*)(GlobalSp)) = f;
-      DEC_DSP
-      STD_IVAL
-      STD_IVAL
+      *((double*)(GlobalFp)) = f;
+      DEC_FSP
   }
   PUSH_IVAL(b)
   return 0;
 }
 /*-------------------------------------------------------------*/
 
+int C_numberquery ()
+{
+  /* stack: ( ^str -- d b | translate characters into number using current base ) */
+
+  char *pStr;
+  long int b, sign, nc;
+
+  b = FALSE;
+  sign = FALSE;
+
+  DROP
+  if (GlobalSp > BottomOfStack) return E_V_STK_UNDERFLOW;
+  CHK_ADDR
+  pStr = *((char**)GlobalSp);
+  PUSH_IVAL(0)
+  PUSH_IVAL(0)
+  nc = *pStr;
+  ++pStr;
+
+  if (*pStr == '-') {
+    sign = TRUE; ++pStr; --nc;
+  }
+  if (nc > 0) {
+        PUSH_ADDR((long int) pStr)
+        PUSH_IVAL(nc)
+        C_tonumber();
+	DROP
+        b = TOS;
+	DROP
+	b = (b == 0) ? TRUE : FALSE ;
+  }
+
+  if (sign) L_dnegate();
+
+  PUSH_IVAL(b)
+  return 0;
+}
+/*----------------------------------------------------------*/
 
 int C_syscall ()
 {
     /* stack: ( arg1 ... arg_n nargs nsyscall -- err | 0 <= n <= 6) */
 
-    int nargs, nsyscall, i, args[6];
-
+    long int nargs, nsyscall, args[6];
+    int i;
     DROP
     nsyscall = TOS; 
     DROP
@@ -1042,7 +1061,7 @@ int C_system ()
   /* stack: ( ^str -- n | n is the return code for the command in ^str ) */
 
   char* cp;
-  int nc, nr;
+  long int nc, nr;
 
   DROP
   CHK_ADDR
@@ -1068,7 +1087,7 @@ int C_chdir ()
   DROP
   CHK_ADDR
   cp = (char*) TOS;
-  nc = *cp;
+  nc = (int) (*cp);
   strncpy (temp_str, cp+1, nc);
   temp_str[nc] = 0;
   PUSH_IVAL( chdir(temp_str) )
@@ -1101,7 +1120,7 @@ int C_usec ()
   /* stack: ( u -- | delay for u microseconds ) */
 
   struct timeval tv1, tv2;
-  unsigned int usec;
+  unsigned long int usec;
 
   DROP
   usec = TOS;
@@ -1154,9 +1173,13 @@ int C_us2fetch ()
   unsigned long long int usec;
   usec = (tv.tv_sec - ForthStartTime.tv_sec)*1000000ULL+
      (tv.tv_usec - ForthStartTime.tv_usec);
-  TOS = *((int*)&usec);
+  TOS = *((long int*)&usec);
   DEC_DSP
-  TOS = *((int*)&usec + 1);
+#if WSIZE == 4
+  TOS = *((long int*)&usec + 1);
+#else
+  TOS = 0;
+#endif
   DEC_DSP
   STD_IVAL
   STD_IVAL
@@ -1164,6 +1187,24 @@ int C_us2fetch ()
   return 0;
 }
 
+void set_start_mem ()
+{
+  /* initialize starting memory usage */
+  ForthStartMem = mallinfo();
+}
+
+int C_used ()
+{
+  /* stack: ( -- u | return bytes used since start of Forth ) */
+  unsigned long u0, u1;
+  struct mallinfo mi = mallinfo();
+  u0 = ForthStartMem.arena + ForthStartMem.hblkhd;
+  u1 = mi.arena + mi.hblkhd;
+  TOS = (u1 - u0);
+  DEC_DSP
+  STD_IVAL
+  return 0;
+}
 /*------------------------------------------------------*/
 
 int C_search ()
@@ -1171,7 +1212,7 @@ int C_search ()
   /* stack: ( a1 u1 a2 u2 -- a3 u3 flag ) */
 
   char *str1, *str2, *cp, *cp2;
-  unsigned int n, n_needle, n_haystack, n_off, n_rem;
+  unsigned long int n, n_needle, n_haystack, n_off, n_rem;
   DROP
   n = TOS;
   DROP
@@ -1218,7 +1259,7 @@ int C_search ()
     ;
 
   if (cp2 == NULL) n_off = 0;
-  TOS = (int)(str1 + n_off);
+  TOS = (long int)(str1 + n_off);
   DEC_DSP
   TOS = n_haystack - n_off;
   DEC_DSP
@@ -1237,7 +1278,7 @@ int C_compare ()
   /* stack: ( a1 u1 a2 u2 -- n ) */
 
   char *str1, *str2;
-  int n1, n2, n, ncmp, nmin;
+  long int n1, n2, n, ncmp, nmin;
   DROP
   n2 = TOS;
   DROP
@@ -1269,7 +1310,7 @@ int C_setitimer ()
 {
     /* stack: ( timer-type avalue aoldvalue -- flag ) */
     
-    int type;
+    long int type;
     struct itimerval *v1, *v2;
 
     DROP
@@ -1288,7 +1329,7 @@ int C_getitimer ()
 {
     /* stack: ( timer-type  avalue -- flag )  */
     
-    int type;
+    long int type;
     struct itimerval *v;
 
     DROP
@@ -1317,7 +1358,7 @@ int C_forth_signal ()
 
     int signum;
     struct sigaction action;
-    void *xt, *oldxt;
+    void **xt, **oldxt;
 
     DROP
     signum = TOS;
@@ -1325,32 +1366,32 @@ int C_forth_signal ()
     {
 	DROP
 	oldxt = signal_xtmap[signum-1];
-        memset( &action, 0, sizeof(struct sigaction));
-        action.sa_flags = SA_SIGINFO;
-	xt = (void *) TOS;
-	switch ((int) xt)
+	memset( &action, 0, sizeof(struct sigaction));
+	action.sa_flags = SA_SIGINFO;
+	xt = (void**) TOS;
+	switch ((long int) xt)
 	{
-	    case (int) SIG_DFL:
+	    case (long int) SIG_DFL:
 		// Reset the default signal handler if xt = 0
-                action.sa_sigaction = (void*) SIG_DFL;
-                sigaction( signum, &action, NULL );
+	        action.sa_sigaction = (void*) SIG_DFL;
+		sigaction( signum, &action, NULL );
 		xt = 0;
 		break;
-	    case (int) SIG_IGN:
+	    case (long int) SIG_IGN:
 		// Ignore the signal if xt = 1
-                action.sa_sigaction = (void*) SIG_IGN;
-                sigaction( signum, &action, NULL );
+	        action.sa_sigaction = (void*) SIG_IGN;
+		sigaction( signum, &action, NULL );
 		xt = 0;
 		break;
 	    default:
 		// All other xt s must be valid addresses to opcodes
 		CHK_ADDR
-                action.sa_sigaction = forth_signal_handler;
-                sigaction( signum, &action, NULL );
+	        action.sa_sigaction = forth_signal_handler;
+		sigaction( signum, &action, NULL );
 		break;
 	}
         signal_xtmap[signum-1] = xt;
-        PUSH_ADDR( (int) oldxt )
+        PUSH_ADDR( (long int) oldxt )
     }
     else
 	return E_V_BAD_OPCODE;
@@ -1372,7 +1413,7 @@ static void forth_signal_handler (int signum, siginfo_t* si, void* vcontext)
        already takes care of preserving and restoring the virtual
        instruction ptr (GlobalIp).
     */
-    int e, *sp = GlobalSp, *rp = GlobalRp;
+    long int e, *sp = GlobalSp, *rp = GlobalRp;
 #ifndef __FAST__ 
     unsigned char* tp = GlobalTp, *rtp = GlobalRtp;
 #endif
@@ -1382,10 +1423,10 @@ static void forth_signal_handler (int signum, siginfo_t* si, void* vcontext)
     ucontext_t* context = (ucontext_t*) vcontext;
 
     // Lookup the execution token of Forth word for this signal.
-    void* xt = signal_xtmap[signum-1];
+    void** xt = signal_xtmap[signum-1];
     if (xt == 0) return;
 
-    opcode = *((byte*) xt);
+    opcode = *((byte*) (*xt));
     if ((opcode == OP_QUIT) || (opcode == OP_ABORT) || (opcode == OP_BYE)) {
       // Handle special signals requiring immediate exit from the VM,
       // without execution of Forth handler, e.g. SIGSEGV
@@ -1394,48 +1435,48 @@ static void forth_signal_handler (int signum, siginfo_t* si, void* vcontext)
         case SIGSEGV:
           msg = "Segmentation fault\n";
           write(1, msg, 19);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         case SIGINT:
-          msg = "Interrupted by user\n";
+	  msg = "Interrupted by user\n";
           write(1, msg, 20);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         case SIGFPE:
           msg = "Floating point exception\n";
           write(1, msg, 25);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         case SIGBUS:
           msg = "Bus error\n";
           write(1, msg, 10);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         case SIGILL:
           msg = "Illegal instruction\n";
           write(1, msg, 20);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         case SIGQUIT:
           msg = "SIGQUIT\n";
           write(1, msg, 8);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         case SIGABRT:
           msg = "SIGABRT\n";
           write(1, msg, 8);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
         default:
           msg = "Signal received\n";
           write(1, msg, 16);
-          context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) pCode;
+          context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) pCode;
           return;
-          break;
+	  break;
        }
     }
 
-    // We must also offset the stack pointers so the handler will not
+    // We must offset the stack pointers so the Forth handler will not
     //   overwrite intermediate stack values in the primary vm(). An offset 
     //   of 16 elements should be safe (worst case is L_utmslash, which
     //   uses about 12 elements above the current stack position for 
@@ -1445,9 +1486,9 @@ static void forth_signal_handler (int signum, siginfo_t* si, void* vcontext)
     GlobalTp -= 16; GlobalRtp -= 16;
 #endif
     PUSH_IVAL(signum);
-    e = vm((byte*) xt);
+    e = vm((byte*) *xt);
     if (e == E_V_QUIT) {
-      context->uc_mcontext.gregs[REG_EIP] = (unsigned long int) L_quit;
+      context->uc_mcontext.gregs[REG_RIP] = (unsigned long int) L_quit;
     }
 
     // Restore data stack and return stack pointers
